/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Collections
*  SCOPE :              Collections common services
*  FILE :               colln.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __colln_HXX
#define __colln_HXX

/* IMPORTS *******************************************************************/

#ifndef __colroot_HXX
#include "colroot.hxx"
#endif

/*****************************************************************************/

class ColLeaf : public Collection {

protected :

  // dummy attribute to be sure that double will be aligned (x8)
  int _dummy ;

  // Use char array to avoid empty construction during refresh phase
  char _tab[sizeof(ColElem)*ColNbInLeaf] ;

  // Return the leaf itself
  Ref<ColLeaf> getLeaf(int) ;

  // Reset leaf-node after insertion of a new child
  void newChild(const void*, int) ;

  // Reset cardinality and number of element after deletion
  void delChild(int) ;

  // Make the leaf grow up. Return the new position of the leaf
  Ref<ColLeaf> grow(int nbe) ;

  // Use to build a new root 
  virtual Ref<ColNode> newNode() = 0 ;  

  // Size of an element of a collection
  int sizeElem() ;

  // Maximum Number of element in a collection leaf or node
  int maxElem() ;

  // Return current number of element in the collection
  int nbElem() ;

  // Insert an element in a sub-tree (without construction of new root)
  Ref<Collection> insertTree(const void* atom, int ind) ;

  // Remove an element from a sub-tree (without deletion of the root)
  void removeTree(int ind) ;
  
public :
  
  ColLeaf() {} ;
  ColLeaf(int card) : Collection(card) {}
  ~ColLeaf() {} ;

  // Apply delete on the leaf
  void       clear() ;

  // Get the nth elem of the collection  
  void* getElem(int ind) ;  

  // Return leaf for scan. Set begin and end for this leaf
  Ref<ColLeaf> scanLeaf(int, int&, int&) ;

  // Return the address of nth element of the leaf
  ColElem* tab(int pos) ;

  // Insert new element in collection. Return the new root
  Ref<Collection> insert(const void*, int) ;

  // Remove the element from the collection
  Ref<Collection> remove(int) ;
} ;


class ColNode : public Collection {

protected :

  // Number of children
  int _nbElem ;
  
  // Return position of the subnode which contains ind. Modify ind
  int node(int& ind) ;
  
  // Return the leaf which contains ind
  Ref<ColLeaf> getLeaf(int ind) ;

  // Reset leaf-node after insertion of a new child
  void newChild(const void*, int) ;

  // Reset cardinality and number of element after deletion
  void delChild(int) ;

  // Reset cardinality and number of element after deletion/insertion
  virtual void resetChild(int) ;

  // Make the node merged
  void merge(int pos) ;

  // Return current number of element in the collection
  int nbElem() ;

  // Insert an element in a sub-tree (without construction of new root)
  Ref<Collection> insertTree(const void* atom, int ind) ;
  
  // Remove an element from a sub-tree (without deletion of the root)
  void removeTree(int ind) ;
  
public :
  
  ColNode() {} ;
  ColNode(int card) : Collection(card), _nbElem(card) {}
  ~ColNode() {} ;

  // Apply delete on the leaf
  void       clear() ;

  // Get the nth elem of the collection  
  void* getElem(int ind) ;  

  // Return leaf for scan. Set begin and end for this leaf
  Ref<ColLeaf> scanLeaf(int, int&, int&) ;

  // Insert new element in collection. Return the new root
  Ref<Collection> insert(const void*, int) ;

  // Remove the element from the collection
  Ref<Collection> remove(int) ;
} ;


#endif
